<?php

declare(strict_types=1);

namespace Dedi\SyliusPayboxPlugin\Payum;

use Dedi\SyliusPayboxPlugin\Payum\Action\CaptureAction;
use Dedi\SyliusPayboxPlugin\Payum\Action\ConvertPaymentAction;
use Dedi\SyliusPayboxPlugin\Payum\Action\NotifyAction;
use Dedi\SyliusPayboxPlugin\Payum\Action\StatusAction;
use Payum\Core\Bridge\Spl\ArrayObject;
use Payum\Core\GatewayFactory;

class PayboxGatewayFactory extends GatewayFactory
{
    protected function populateConfig(ArrayObject $config)
    {
        $config->defaults([
            'payum.factory_name' => 'paybox',
            'payum.factory_title' => 'Paybox',

            'payum.service.requestStack' => '@request_stack',
            'payum.service.sm' => '@sm.factory',

            'payum.action.capture' => static function (ArrayObject $config) {
                return new CaptureAction($config['payum.service.requestStack']);
            },
            'payum.action.notify' => static function (ArrayObject $config) {
                return new NotifyAction($config['payum.service.requestStack'], $config['payum.service.sm']);
            },
            'payum.action.status' => static function (ArrayObject $config) {
                return new StatusAction($config['payum.service.requestStack']);
            },
            'payum.action.convert_payment' => new ConvertPaymentAction(),
        ]);

        if (false == $config['payum.api']) {
            $config['payum.default_options'] = [
                'site' => '',
                'rang' => '',
                'identifiant' => '',
                'hmac' => '',
                'hash' => 'SHA512',
                'retour' => 'Mt:M;Ref:R;Auto:A;Appel:T;Abo:B;Reponse:E;Transaction:S;Pays:Y;Signature:K',
                'sandbox' => true,
                'type_paiement' => '',
                'type_carte' => '',
            ];
            $config->defaults($config['payum.default_options']);
            $config['payum.required_options'] = ['site', 'rang', 'identifiant', 'hmac'];

            $config['payum.api'] = static function (ArrayObject $config) {
                $config->validateNotEmpty($config['payum.required_options']);

                return new PayboxApi((array) $config, $config['payum.http_client'], $config['httplug.message_factory']);
            };
        }
    }
}
